@@ -35,6 +35,7 @@
 #endif
 
 #include <dbus/dbus-glib.h>
+#include <mce/dbus-names.h>
 
 #if HILDON == 1
 #include <hildon/hildon.h>
@@ -47,7 +48,6 @@
 void safeReplaceString(char** oldstr, const gchar *newstr, MainView *mainview);
 void maybe_save_file(MainView *mainview);
 void save_mod_file(MainView *mainview);
-char *get_confirmed_pass(MainView *mainview);
 
 /* activate grid */
 void callback_activate_treeView(GtkTreeView *treeView, GtkTreePath *path, GtkTreeViewColumn *column, MainView *mainview) {
@@ -69,7 +69,7 @@ void callback_hildon_activate_treeView(GtkTreeView *treeView, GtkTreePath *path,
 	onrec=g_object_get_data (G_OBJECT(&iter), "r");
 	*/
 	if (onrec!=NULL) {
-		show_rec(mainview, onrec);
+		show_rec(mainview, onrec, FALSE);
 	} else if (gtk_tree_view_collapse_row(treeView, path)) {
 		gtk_tree_view_expand_row(treeView, path, FALSE);
 	} else {
@@ -109,15 +109,11 @@ void safeReplaceString(char** oldstr, const gchar *newstr, MainView *mainview) {
 }
 
 const gchar *entryGetText(GtkWidget *widget) {
-#if MAEMO_VERSION_MAJOR < 5
 	return gtk_entry_get_text(GTK_ENTRY(widget));
-#else
-	return hildon_entry_get_text(HILDON_ENTRY(widget));
-#endif
 }
 
 /* record detail events */
-void callback_rec_response(GtkDialog *dialog, gint arg1, MainView *mainview) {
+void callback_rec_response(GtkDialog *dialog, gint response, MainView *mainview) {
 	RecWindow *recwindow = NULL;
 	rec *rec=NULL;
 	GtkTextBuffer *buffer=NULL;
@@ -128,18 +124,17 @@ void callback_rec_response(GtkDialog *dialog, gint arg1, MainView *mainview) {
 	
 	recwindow = mainview->rec_win;
 	rec=recwindow->rec;
-	switch (arg1) {
+	switch (response) {
 		case 1: /* save any changes */
 			safeReplaceString(&rec->title, entryGetText(recwindow->title), mainview);
 			safeReplaceString(&rec->user, entryGetText(recwindow->username), mainview);
 			safeReplaceString(&rec->password, entryGetText(recwindow->password), mainview);
 #if MAEMO_VERSION_MAJOR < 5
 			safeReplaceString(&rec->group, gtk_combo_box_get_active_text(GTK_COMBO_BOX(recwindow->group)), mainview);
-			buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW(recwindow->notes));
 #else
 			safeReplaceString(&rec->group, hildon_button_get_value(HILDON_BUTTON(recwindow->group)), mainview);
-			buffer=hildon_text_view_get_buffer(HILDON_TEXT_VIEW(recwindow->notes));
 #endif
+			buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW(recwindow->notes));
 			if (gtk_text_buffer_get_modified(buffer)) { /* update notes if buffer has changed */
 				gtk_text_buffer_get_start_iter (buffer, &start);
 				gtk_text_buffer_get_end_iter (buffer, &end);
@@ -165,9 +160,12 @@ void callback_rec_response(GtkDialog *dialog, gint arg1, MainView *mainview) {
 		case 5:
 			interface_set_rec_editable(recwindow, TRUE);
 			break;
-		case -4:
-			//Cancel fro Fremantle (out of dialog click)
+		default:
+			//Cancel for Fremantle (out of dialog click)
 			unshow_rec(mainview);  /* close rec window */
+			if (recwindow->create) {
+				delete_rec(rec);
+			}
 			populate_treeView(mainview); /* redraw grid */
 			break;
 	}
@@ -184,8 +182,7 @@ void setUrlTag(GtkTextBuffer *buffer, gint posStart, gint posEnd) {
 #if MAEMO_VERSION_MAJOR < 5
 void enhanceBufferWithString(GtkTextBuffer *buffer, const gchar *search) {
 	GtkTextIter start, end;
-	gchar *text, *bout, *oneChar;
-	gunichar uniChar;
+	gchar *text, *bout;
 	gint posStart, posEnd, index, listIdx;
 	gchar **list, **searchList;
 	
@@ -316,9 +313,9 @@ void hw_event_handler(osso_hw_state_t *state, MainView *mainview) {
 
 	if ((state->shutdown_ind)	/* system shutdown */
 			|| (state->system_inactivity_ind)) {	/* system sleeping */
-        save_mod_file(mainview);
+		save_mod_file(mainview);
 		close_file(mainview);
-        return;
+		return;
 	}
 	if (state->memory_low_ind) {	/* memory low... */
 		unshow_rec(mainview);	/* close the rec window... changes not saved... */
@@ -379,57 +376,38 @@ gchar * getCurrentGroup(MainView *mainview ) {
 void callback_add_entry ( GtkAction * action, MainView *mainview ) {
 	rec* newrec=NULL;
 	g_assert(mainview != NULL && mainview->data != NULL );
-	newrec=add_new_record("new", getCurrentGroup(mainview));
+	newrec=add_new_record("", getCurrentGroup(mainview));
 	populate_treeView(mainview);
-	show_rec(mainview, newrec);
-    interface_set_rec_editable(mainview->rec_win, TRUE);
+	show_rec(mainview, newrec, TRUE);
+	interface_set_rec_editable(mainview->rec_win, TRUE);
 }
 
 /* change password */
 void callback_change_password(GtkAction * action, MainView *mainview) {
-	char *oldpass=NULL;
+	char *newpass=NULL;
 	g_assert(mainview != NULL && mainview->data != NULL );
-	oldpass=get_password(mainview, "Old Password");
-	if (oldpass && mainview->pass && (strcmp(oldpass, mainview->pass)==0)) {
-		char *newpass=get_confirmed_pass(mainview);
-		if (newpass) mainview->pass=newpass;
+	newpass=get_password(mainview, "Change Password", PWD_DLG_CHANGE);
+	if (newpass) {
+		mainview->pass=newpass;
+		mainview->file_edited = TRUE;
 	}
 }
 
-char *get_confirmed_pass(MainView *mainview) {
-	char *result=NULL;
-	char *confirm=NULL;
-	result=get_password(mainview, "New Password");
-	if (result==NULL) return NULL;
-	confirm=get_password(mainview, "Confirm Password");
-	if (confirm==NULL) {
-		trashFreeMemory(result, -1);
-		return NULL;
-	}
-	if (strcmp(result, confirm)==0) {
-		trashFreeMemory(confirm, -1);
-		return result;
-	}
-	trashFreeMemory(confirm, -1);
-	trashFreeMemory(result, -1);
-	msgbox(mainview, "Password Mismatch");
-	return NULL;
-}
-
 /* new */
 void callback_file_new(GtkAction * action, MainView *mainview) {
 	char *newpass;
 	g_assert(mainview != NULL && mainview->data != NULL );
 	/* maybe_save_file(mainview); */
-    save_mod_file(mainview);
+	save_mod_file(mainview);
 	close_file(mainview);
-	newpass=get_confirmed_pass(mainview);
+	newpass=get_password(mainview, "New Password", PWD_DLG_NEW);
 	if (newpass) {
 		mainview->pass=newpass;
 		add_new_record("new", getCurrentGroup(mainview));
 		populate_treeView(mainview);
-        /* maybe_save_file(mainview); */
-        write_pwsafe_file(mainview);  /* force set file name */
+		/* maybe_save_file(mainview); */
+		write_pwsafe_file(mainview);  /* force set file name */
+		interface_show_state(mainview) ;
 	}
 }
 
@@ -477,6 +455,15 @@ void callback_fullscreen( GtkAction * action, gpointer data ) {
 	interface_fullscreen((MainView *) data);
 }
 
+gint searchicase(const char *str1, const char *str2) {
+	if (str1 == NULL) {
+		return 1;
+	}
+	if (str2 == NULL) {
+		return 0;
+	}
+	return strcasestr(str1, str2)? 0: 1;
+}
 
 /* filter */
 
@@ -484,13 +471,7 @@ void callback_filter(GtkWidget *widget, MainView *mainview ) {
     const gchar *ftxt = NULL;
     GList *onrec=NULL;
 
-#if MAEMO_VERSION_MAJOR < 5
-	ftxt = gtk_entry_get_text(GTK_ENTRY(widget));
-#else
-	ftxt = hildon_entry_get_text(HILDON_ENTRY(widget));
-#endif
-//    g_object_get(G_OBJECT(action), "text", &ftxt, NULL);
-	g_printerr("filter -%s-\n", ftxt);
+    ftxt = gtk_entry_get_text(GTK_ENTRY(widget));
     
     onrec=g_list_first(recs);
     if (ftxt[0] == '\0') { /* no filter, skip= none */
@@ -503,15 +484,15 @@ void callback_filter(GtkWidget *widget, MainView *mainview ) {
         while (onrec!=NULL) {
             rec *data=(rec *) onrec->data;
             
-            data->skip= ( strcasestr(data->group,ftxt) ? 0 : 1 );
+            data->skip= searchicase(data->group,ftxt);
             if (data->skip != 0) {
-                data->skip= ( strcasestr(data->title,ftxt) ? 0 : 1);
+                data->skip= searchicase(data->title,ftxt);
                 if (data->skip != 0) {
-                    data->skip= ( strcasestr(data->user,ftxt) ? 0 : 1);
+                    data->skip= searchicase(data->user,ftxt);
                     if (data->skip != 0) {
-                        data->skip= ( strcasestr(data->notes,ftxt) ? 0 : 1);
+                        data->skip= searchicase(data->notes,ftxt);
                         if (data->skip != 0) {
-                            data->skip= ( strcasestr(data->password,ftxt) ? 0 : 1);
+                            data->skip= searchicase(data->password,ftxt);
                         }
                     }
                 }
@@ -522,6 +503,120 @@ void callback_filter(GtkWidget *widget, MainView *mainview ) {
     populate_treeView(mainview);
 }
 
+gint getEntryLength(GtkEntry *entry) {
+#if MAEMO_VERSION_MAJOR < 5
+	gchar *text;
+	text = gtk_entry_get_text(entry);
+	return (text == NULL)?0:strlen(text);
+#else
+	return gtk_entry_get_text_length(entry);
+#endif	
+}
+
+gboolean isTextEqual(const gchar *text1, const gchar *text2) {
+	if (text1 == NULL && text2 == NULL) {
+		return TRUE;
+	} else if (text1 == NULL || text2 == NULL) {
+		return FALSE;
+	}
+#if MAEMO_VERSION_MAJOR < 5
+	return (strcmp(text1, text2) == 0);
+#else
+	return (g_strcmp0(text1, text2) == 0);
+#endif	
+}
+
+/* password Dialog */
+void callback_pwdialog_changed(GtkWidget *widget, PasswordUI *pwUI) {
+	gboolean activate = FALSE;
+	gint len;
+	if (pwUI->type == PWD_DLG_GET) {
+		activate = (getEntryLength(GTK_ENTRY(widget)) != 0);
+	} else if (pwUI->type == PWD_DLG_NEW) {
+		len = getEntryLength(GTK_ENTRY(pwUI->entry1));
+		activate = (len != 0 && getEntryLength(GTK_ENTRY(pwUI->entry2)) == len);
+		if (activate) {
+			activate = isTextEqual(gtk_entry_get_text(GTK_ENTRY(pwUI->entry1)), gtk_entry_get_text(GTK_ENTRY(pwUI->entry2)));
+		}
+		if (!activate) {
+			g_free(pwUI->msg);
+			if (len == 0) {
+				pwUI->msg = g_strdup(_("Empty Password Not Allowed"));
+			} else {
+				pwUI->msg = g_strdup(_("Password Mismatch"));
+			}
+		}
+	} else {
+		activate = (getEntryLength(GTK_ENTRY(pwUI->entry1)) != 0);
+		if (activate) {
+			len = getEntryLength(GTK_ENTRY(pwUI->entry2));
+			activate = (len != 0 && getEntryLength(GTK_ENTRY(pwUI->entry3)) == len);
+			if (activate) {
+				activate = isTextEqual(gtk_entry_get_text(GTK_ENTRY(pwUI->entry2)), gtk_entry_get_text(GTK_ENTRY(pwUI->entry3)));
+			}
+			if (!activate) {
+				g_free(pwUI->msg);
+				if (len == 0) {
+					pwUI->msg = g_strdup(_("Empty Password Not Allowed"));
+				} else {
+					pwUI->msg = g_strdup(_("Password Mismatch"));
+				}
+			}
+		} else {
+			g_free(pwUI->msg);
+			pwUI->msg = g_strdup(_("Empty Password Not Allowed"));
+		}
+	} 
+	gtk_widget_set_sensitive(pwUI->button, activate);
+}
+
+void callback_pwdialog_activate(GtkWidget *widget, PasswordUI *pwUI) {
+	if (GTK_WIDGET_IS_SENSITIVE(pwUI->button)) {
+		gtk_button_clicked(GTK_BUTTON(pwUI->button));
+	} else if (pwUI->msg != NULL) {
+		msgbox(pwUI->main, pwUI->msg);
+		g_free(pwUI->msg);
+		pwUI->msg = NULL;
+	}
+}
+
+void callback_pwdialog_cancel(GtkWidget *widget, PasswordUI *pwUi) {
+	gtk_dialog_response(GTK_DIALOG(pwUi->dialog), GTK_RESPONSE_CANCEL);
+}
+
+void callback_pwdialog_button(GtkWidget *widget, PasswordUI *pwUi) {
+	if (pwUi->type == PWD_DLG_GET) {
+		pwUi->result=g_strdup(gtk_entry_get_text(GTK_ENTRY(pwUi->entry1)));
+	} else if (pwUi->type == PWD_DLG_NEW) {
+		pwUi->result=g_strdup(gtk_entry_get_text(GTK_ENTRY(pwUi->entry1)));
+		if (!isTextEqual(pwUi->result, gtk_entry_get_text(GTK_ENTRY(pwUi->entry2)))) {
+			msgbox(pwUi->main, _("Password Mismatch"));
+			g_free(pwUi->result);
+//				trashFreeMemory(result, -1);
+			pwUi->result = NULL;
+		}
+	} else {
+		pwUi->result=g_strdup(gtk_entry_get_text(GTK_ENTRY(pwUi->entry1)));
+		if (!isTextEqual(pwUi->result, pwUi->main->pass)) {
+			msgbox(pwUi->main, _("Invalid Password"));
+			g_free(pwUi->result);
+//				trashFreeMemory(result, -1);
+			pwUi->result = NULL;
+		} else {
+			pwUi->result=g_strdup(gtk_entry_get_text(GTK_ENTRY(pwUi->entry2)));
+			if (!isTextEqual(pwUi->result, gtk_entry_get_text(GTK_ENTRY(pwUi->entry3)))) {
+				msgbox(pwUi->main, _("Password Mismatch"));
+				g_free(pwUi->result);
+//					trashFreeMemory(result, -1);
+				pwUi->result = NULL;
+			}
+		}
+	}
+	if (pwUi->result != NULL) {
+		gtk_dialog_response(GTK_DIALOG(pwUi->dialog), GTK_RESPONSE_OK);
+	}
+}
+
 /* export */
 void callback_export ( GtkAction * action, MainView *mainview ) {
     char *fname;
@@ -551,3 +646,19 @@ void callback_export ( GtkAction * action, MainView *mainview ) {
         }
     }
 }
+
+DBusHandlerResult dbus_handle_mce_message(DBusConnection *con, DBusMessage *msg, MainView *mainView ) {
+	DBusMessageIter iter;
+	dbus_bool_t value;
+
+	if (dbus_message_is_signal(msg, MCE_SIGNAL_IF, MCE_INACTIVITY_SIG)) {
+		if (dbus_message_iter_init(msg, &iter)) {
+			dbus_message_iter_get_basic(&iter, &value);
+			if (value) {
+				interfaceCallLastItem(mainView) ;
+			}
+			g_printerr("INFO: New activity status is now: %d\n", value);
+		}
+	}
+	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
