@@ -46,6 +46,7 @@
 #include <hildon-widgets/hildon-get-password-dialog.h>
 #endif
 
+#include <mce/dbus-names.h>
 
 #define MAIN_TEXT MAIN_VIEW_NAME
 
@@ -92,10 +93,15 @@ MainView* interface_main_view_new( AppData *data ) {
 	gtk_widget_grab_focus( GTK_WIDGET(result->treeView ));
 	/* attach handlers */
 	osso_hw_set_event_cb( data->osso, NULL,(osso_hw_cb_f *) hw_event_handler, result);
-//	osso_application_set_exit_cb( data->osso, exit_event_handler, result);
 	g_signal_connect (G_OBJECT(data->main_view), "delete_event", G_CALLBACK(exit_event_handler), result);	/* top right X, etc. */
 	g_signal_connect(G_OBJECT(result->data->main_view), "key_press_event", G_CALLBACK(callback_key_press), result);
-	
+
+	/* Get the system DBus connection */
+	DBusConnection *con = osso_get_sys_dbus_connection(data->osso);
+	/* Add the callback, which should be called, once the device is rotated */
+	dbus_bus_add_match(con, MCE_MATCH_RULE, NULL);
+	dbus_connection_add_filter(con, (DBusHandleMessageFunction *) dbus_handle_mce_message, result, NULL);
+
 #if MAEMO_VERSION_MAJOR >= 5
 	result->regexHttp = g_regex_new("http(s?)://[\\S]+", G_REGEX_CASELESS, 0, NULL);
 	result->regexWww = g_regex_new("www\\.[\\S]+", G_REGEX_CASELESS, 0, NULL);
@@ -116,16 +122,22 @@ void interface_main_view_destroy( MainView *main ) {
 GtkWidget *show_rec_line(GtkWidget *sizer, char *label, char *txt, int row, int col) {
 	GtkWidget *textentry;
 #if MAEMO_VERSION_MAJOR < 5
-	gtk_table_attach_defaults(GTK_TABLE(sizer), gtk_label_new(label), col, col+1, row, row+1);
+	if (label != NULL) {
+		gtk_table_attach_defaults(GTK_TABLE(sizer), gtk_label_new(label), col, col+1, row, row+1);
+	}
 	textentry=gtk_entry_new();
-	if (txt!=NULL) gtk_entry_set_text(GTK_ENTRY(textentry), txt);
 	gtk_table_attach_defaults(GTK_TABLE(sizer), textentry, col+1,col+2, row, row+1);
 #else
 	textentry=hildon_entry_new(HILDON_SIZE_AUTO);
-	hildon_entry_set_placeholder(HILDON_ENTRY(textentry), label);
-	if (txt!=NULL) hildon_entry_set_text(HILDON_ENTRY(textentry), txt);
+	if (label != NULL) {
+		hildon_entry_set_placeholder(HILDON_ENTRY(textentry), label);
+	}
 	gtk_table_attach_defaults(GTK_TABLE(sizer), textentry, col,col+2, row, row+1);
 #endif
+	hildon_gtk_entry_set_input_mode(GTK_ENTRY(textentry), HILDON_GTK_INPUT_MODE_FULL);
+	if (txt!=NULL) {
+		gtk_entry_set_text(GTK_ENTRY(textentry), txt);
+	}
 	return textentry;
 }
 
@@ -137,20 +149,85 @@ void unshow_rec(MainView *main) {
 	}
 }
 
-gchar *get_password(MainView *main, gchar *caption) {
+gchar *get_password(MainView *main, gchar *caption, gint pwdDlgType) {
+	PasswordUI *pwUi;
 	gchar *result=NULL;
-	GtkWidget*  pwdialog;
-	pwdialog = hildon_get_password_dialog_new(GTK_WINDOW(main->data->main_view), FALSE);
-#if HILDON == 1
-	hildon_get_password_dialog_set_message (HILDON_GET_PASSWORD_DIALOG(pwdialog), caption);
+#if MAEMO_VERSION_MAJOR >= 5
+	GtkWidget *image;
 #else
-	g_object_set(pwdialog, "domain", caption, NULL);
+	GtkWidget *cancelButton;
 #endif
-	gtk_widget_show(GTK_WIDGET(pwdialog));
-	if (GTK_RESPONSE_OK==gtk_dialog_run(GTK_DIALOG(pwdialog))) {
-		result=g_strdup(hildon_get_password_dialog_get_password(HILDON_GET_PASSWORD_DIALOG(pwdialog)));
+	GtkWidget *contentArea;
+	GtkWidget *actionArea;
+	GtkWidget *table;
+
+	pwUi = g_new0(PasswordUI, 1);
+	pwUi->type = pwdDlgType;
+	pwUi->main = main;
+	pwUi->result = NULL;
+	pwUi->msg = NULL;
+	
+	pwUi->dialog = gtk_dialog_new_with_buttons(caption, GTK_WINDOW(main->data->main_view), GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR, NULL);
+	table = gtk_table_new(pwdDlgType, 2, TRUE);
+
+#if MAEMO_VERSION_MAJOR < 5
+	actionArea = GTK_DIALOG(pwUi->dialog)->action_area;
+	contentArea = GTK_DIALOG(pwUi->dialog)->vbox;
+
+	pwUi->button = gtk_button_new_from_stock(GTK_STOCK_OK);
+	cancelButton = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+#else
+	actionArea = gtk_dialog_get_action_area(GTK_DIALOG(pwUi->dialog));
+	contentArea = gtk_dialog_get_content_area(GTK_DIALOG(pwUi->dialog));
+
+	pwUi->button =  hildon_button_new_with_text(HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL, _("OK"), NULL);
+	image = gtk_image_new_from_stock (GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON);
+	hildon_button_set_image (HILDON_BUTTON (pwUi->button), image);
+	pwUi->entry1=hildon_entry_new(HILDON_SIZE_AUTO);
+#endif
+	gtk_container_add (GTK_CONTAINER (actionArea), pwUi->button);
+#if MAEMO_VERSION_MAJOR < 5
+	gtk_container_add (GTK_CONTAINER (actionArea), cancelButton);
+	g_signal_connect (cancelButton, "clicked", G_CALLBACK (callback_pwdialog_cancel), pwUi);
+#endif
+	gtk_container_add (GTK_CONTAINER (contentArea), table);
+
+	gtk_widget_set_sensitive(pwUi->button, FALSE);
+	g_signal_connect (pwUi->button, "clicked", G_CALLBACK (callback_pwdialog_button), pwUi);
+
+	if (pwdDlgType != PWD_DLG_GET) {
+		if (pwdDlgType != PWD_DLG_NEW) {
+			pwUi->entry1=show_rec_line(table, _("Old password"), NULL, 0, 0);
+			pwUi->entry2=show_rec_line(table,  _("New password"), NULL, 1, 0);
+			pwUi->entry3=show_rec_line(table, _("Confirm new password"), NULL, 2, 0);
+			gtk_entry_set_visibility(GTK_ENTRY(pwUi->entry3), FALSE);
+			g_signal_connect(G_OBJECT(pwUi->entry3), "changed", G_CALLBACK(callback_pwdialog_changed), pwUi);
+			g_signal_connect(G_OBJECT(pwUi->entry3), "activate", G_CALLBACK(callback_pwdialog_activate), pwUi);
+		} else {
+			pwUi->entry1=show_rec_line(table,  _("Password"), NULL, 0, 0);
+			pwUi->entry2=show_rec_line(table, _("Confirm password"), NULL, 1, 0);
+		}
+		gtk_entry_set_visibility(GTK_ENTRY(pwUi->entry2), FALSE);
+		g_signal_connect(G_OBJECT(pwUi->entry2), "changed", G_CALLBACK(callback_pwdialog_changed), pwUi);
+		g_signal_connect(G_OBJECT(pwUi->entry2), "activate", G_CALLBACK(callback_pwdialog_activate), pwUi);
+	} else {
+		pwUi->entry1=show_rec_line(table, _("Password"), NULL, 0, 0);
 	}
-	gtk_widget_destroy (GTK_WIDGET (pwdialog));
+	gtk_entry_set_visibility(GTK_ENTRY(pwUi->entry1), FALSE);
+	g_signal_connect(G_OBJECT(pwUi->entry1), "changed", G_CALLBACK(callback_pwdialog_changed), pwUi);
+	g_signal_connect(G_OBJECT(pwUi->entry1), "activate", G_CALLBACK(callback_pwdialog_activate), pwUi);
+
+	gtk_widget_show_all(GTK_WIDGET(pwUi->dialog));
+	if (GTK_RESPONSE_OK == gtk_dialog_run(GTK_DIALOG(pwUi->dialog))) {
+		// Getting out of the password Dialog
+		if (pwUi->result != NULL) {
+			result = g_strdup(pwUi->result);
+			g_free(pwUi->result);
+		}
+	}
+	
+	g_free(pwUi);
+	gtk_widget_destroy (GTK_WIDGET (pwUi->dialog));
 	return result;
 }
 
@@ -169,7 +246,7 @@ gboolean fillCombo (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, G
 	return FALSE;
 }
 
-void show_rec(MainView *main, rec *r) {
+void show_rec(MainView *main, rec *r, gboolean create) {
 	GtkWidget *sizer, *scrollwindow;
 #if MAEMO_VERSION_MAJOR >= 5
 	GtkWidget *selector;
@@ -180,12 +257,14 @@ void show_rec(MainView *main, rec *r) {
 	if (main->rec_win!=NULL) return; /* don't show two rec windows at once */
 	rec_win=g_malloc0(sizeof(RecWindow));
 	main->rec_win=rec_win;
+	rec_win->create=create;
 	rec_win->rec=r;
 	rec_win->dialog = gtk_dialog_new_with_buttons (_("Edit/View Entry"), GTK_WINDOW(main->data->main_view), GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR, GTK_STOCK_OK, 1, _("Copy User"), 2, _("Copy Pwd"), 3, NULL);
 	rec_win->delete_button = gtk_dialog_add_button(GTK_DIALOG(rec_win->dialog), GTK_STOCK_DELETE, 4);
 	rec_win->edit_button = gtk_dialog_add_button(GTK_DIALOG(rec_win->dialog), _("Edit"), 5);
 /*	gtk_button_set_image(GTK_BUTTON(rec_win->edit_button), GTK_STOCK_EDIT); */
 #if MAEMO_VERSION_MAJOR < 5
+	gtk_dialog_add_button(GTK_DIALOG(rec_win->dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	g_object_set(rec_win->dialog, "content-area-border", 1, "button_spacing", 20, NULL);
 #endif
     /*  where's "button-spacing", "content-area-border", "action_area_border" ? - not in scratchbox, maybe on n800 */
@@ -210,7 +289,7 @@ void show_rec(MainView *main, rec *r) {
 	selector=hildon_touch_selector_entry_new_text();
 	hildon_entry_set_placeholder(hildon_touch_selector_entry_get_entry(HILDON_TOUCH_SELECTOR_ENTRY(selector)), _("Group"));
 	if (r->group!=NULL) {
-		hildon_entry_set_text(hildon_touch_selector_entry_get_entry(HILDON_TOUCH_SELECTOR_ENTRY(selector)), r->group);
+		gtk_entry_set_text(GTK_ENTRY(hildon_touch_selector_entry_get_entry(HILDON_TOUCH_SELECTOR_ENTRY(selector))), r->group);
 	}
 	hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (rec_win->group), HILDON_TOUCH_SELECTOR (selector));
 
@@ -534,6 +613,9 @@ void interface_show_state(MainView *main) {
 		gtk_widget_set_sensitive(GTK_WIDGET(main->change_password_tb), fopen);
 		gtk_widget_set_sensitive(GTK_WIDGET(main->add_tb), fopen);
 		gtk_editable_set_editable(GTK_EDITABLE(main->filter_tb), fopen);
+		if (!fopen) {
+			gtk_entry_set_text(GTK_ENTRY(main->filter_tb), "");
+		}
 	}
 }
 
@@ -679,3 +761,11 @@ void interface_fullscreen(MainView *mainview) {
 	}
 }
 
+void interfaceCallLastItem(MainView *mainView) {
+#if MAEMO_VERSION_MAJOR < 5
+	g_signal_emit_by_name(G_OBJECT(mainView->open_last_item), "activate");
+#else
+	g_signal_emit_by_name(G_OBJECT(mainView->open_last_item), "clicked");
+#endif
+}
+
